@@ -107,6 +107,10 @@ public:
 	 */
 	virtual void send_message(const std::string& message);
 
+	/** Sends a message to the given user.
+	 */
+	virtual void send_message(const std::string& message, const user& to);
+
 	/** @brief Sets whether keepalives are sent to clients.
 	 *
 	 * With this option enabled, the server sends keepalive packets to
@@ -171,6 +175,13 @@ protected:
 	void send_message_impl(const std::string& message,
 	                       const obby::user* writer);
 
+	/** Internal function send a message to a given user that comes from
+	 * the user <em>writer</em>.
+	 */
+	void send_message_impl(const std::string& message,
+	                       const obby::user* writer,
+			       const obby::user& to);
+
 	/** Changes the colour of the given user to the new provided
 	 * colour and relays the fact to the other users.
 	 */
@@ -251,6 +262,8 @@ protected:
 
 	command_map m_command_map;
 private:
+	void reopen_impl(unsigned int port);
+
 	/** This function provides access to the underlaying net6::basic_server
 	 * object.
 	 */
@@ -281,6 +294,28 @@ basic_server_buffer<Document, Selector>::
 	);
 }
 
+template<typename Document, typename Selector>
+void basic_server_buffer<Document, Selector>::reopen_impl(unsigned int port)
+{
+	if(IPV6_ENABLED)
+	{
+		// Prefer IPv6 when compiled with IPv6 support,
+		// fallback to IPv4
+		try
+		{
+			net6_server().reopen(port, true);
+		}
+		catch(net6::error& e)
+		{
+			net6_server().reopen(port, false);
+		}
+	}
+	else
+	{
+		net6_server().reopen(port, false);
+	}
+}
+
 template<typename Document, typename Selector>
 void basic_server_buffer<Document, Selector>::open(unsigned int port)
 {
@@ -295,7 +330,7 @@ void basic_server_buffer<Document, Selector>::open(unsigned int port)
 	basic_buffer<Document, Selector>::m_net.reset(new_net());
 	register_signal_handlers();
 
-	net6_server().reopen(port);
+	reopen_impl(port);
 
 	// Clear previous documents and users
 	basic_buffer<Document, Selector>::document_clear();
@@ -322,7 +357,7 @@ void basic_server_buffer<Document, Selector>::open(const std::string& session,
 	basic_buffer<Document, Selector>::m_net.reset(new_net());
 	register_signal_handlers();
 
-	net6_server().reopen(port);
+	reopen_impl(port);
 
 	// Deserialise file
 	serialise::parser parser;
@@ -478,6 +513,14 @@ void basic_server_buffer<Document, Selector>::
 	send_message_impl(message, NULL);
 }
 
+template<typename Document, typename Selector>
+void basic_server_buffer<Document, Selector>::
+	send_message(const std::string& message, const user& to)
+{
+	// send_message_impl relays the message to the clients
+	send_message_impl(message, NULL, to);
+}
+
 template<typename Document, typename Selector>
 void basic_server_buffer<Document, Selector>::
 	set_enable_keepalives(bool enable)
@@ -576,6 +619,32 @@ void basic_server_buffer<Document, Selector>::
 	}
 }
 
+template<typename Document, typename Selector>
+void basic_server_buffer<Document, Selector>::
+	send_message_impl(const std::string& message, const user* writer, const user& to)
+{
+	if(basic_buffer<Document, Selector>::is_open())
+	{
+		net6::packet message_pack("obby_message");
+		message_pack << writer << message;
+		net6_server().send(message_pack, to.get_net6());
+	}
+
+	if(writer == NULL)
+	{
+		basic_buffer<Document, Selector>::m_chat.add_server_message(
+			message
+		);
+	}
+	else
+	{
+		basic_buffer<Document, Selector>::m_chat.add_user_message(
+			message,
+			*writer
+		);
+	}
+}
+
 template<typename Document, typename Selector>
 void basic_server_buffer<Document, Selector>::
 	send_message_impl(const std::string& message, const user* writer)
@@ -1173,7 +1242,7 @@ template<typename Document, typename Selector>
 typename basic_server_buffer<Document, Selector>::base_net_type*
 basic_server_buffer<Document, Selector>::new_net()
 {
-	return new net_type(IPV6_ENABLED);
+	return new net_type();
 }
 
 template<typename Document, typename Selector>
